"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.CachedAsync = CachedAsync;
const tslib_1 = require("tslib");
const common_1 = require("@nestjs/common");
const constants_1 = require("../constants");
const utils_1 = require("../utils");
const logger = new common_1.Logger('LruCache', { timestamp: true });
function createCachedAsyncFunction(target, propertyKey, origFn, options) {
    return function (...args) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            var _a, _b;
            if (!this[constants_1.CACHE_INSTANCE]) {
                logger.warn(`Failed to get the cache instance in method ${target.constructor.name}.${String(propertyKey)}(). This may be because the class using the @CachedAsync decorator has not been registered as a provider in the NestJS module, and therefore is not available in the DI container. This method's results will not be cached.`);
                return yield origFn.apply(this, args);
            }
            const cacheInstanceId = this[constants_1.CACHE_INSTANCE_ID_PROPERTY] ? `_${this[constants_1.CACHE_INSTANCE_ID_PROPERTY]}` : '';
            let cacheOptionsArg;
            if (options.useArgumentOptions && args.length > 0) {
                const possibleCacheOptions = args.at(-1);
                if ((0, utils_1.isObject)(possibleCacheOptions)) {
                    cacheOptionsArg = possibleCacheOptions;
                }
            }
            const mergedOptions = Object.assign(Object.assign({}, options), cacheOptionsArg);
            let cacheKey = mergedOptions.useSharedCache
                ? `${this.constructor.name}.${String(propertyKey)}`
                : `${this.constructor.name}${cacheInstanceId}.${String(propertyKey)}`;
            if (options.hashFunction) {
                cacheKey += `:${options.hashFunction.apply(this, args)}`;
            }
            cacheKey = (0, utils_1.wrapCacheKey)(cacheKey);
            if (!mergedOptions.ignoreCached && this[constants_1.CACHE_INSTANCE].has(cacheKey, mergedOptions)) {
                const cachedVal = this[constants_1.CACHE_INSTANCE].get(cacheKey, mergedOptions);
                if (cachedVal instanceof Promise) {
                    return (yield cachedVal);
                }
                return cachedVal;
            }
            const originalPromise = origFn.apply(this, args);
            if ((_a = mergedOptions.cachePromise) !== null && _a !== void 0 ? _a : true) {
                this[constants_1.CACHE_INSTANCE].set(cacheKey, originalPromise, mergedOptions);
            }
            let result;
            try {
                result = yield originalPromise;
            }
            catch (e) {
                if (mergedOptions.deleteRejectedPromise) {
                    this[constants_1.CACHE_INSTANCE].delete(cacheKey);
                }
                throw e;
            }
            if ((_b = mergedOptions.cachePromiseResult) !== null && _b !== void 0 ? _b : true) {
                this[constants_1.CACHE_INSTANCE].set(cacheKey, result, mergedOptions);
            }
            else {
                this[constants_1.CACHE_INSTANCE].delete(cacheKey);
            }
            return result;
        });
    };
}
function CachedAsync(optionsOrHashFunctionOrTtl = {}) {
    const injectCache = (0, common_1.Inject)(constants_1.LRU_CACHE);
    return (target, propertyKey, descriptor) => {
        injectCache(target, constants_1.CACHE_INSTANCE);
        let options = {};
        if (typeof optionsOrHashFunctionOrTtl === 'object') {
            options = optionsOrHashFunctionOrTtl;
        }
        else if (typeof optionsOrHashFunctionOrTtl === 'number') {
            options.ttl = optionsOrHashFunctionOrTtl;
        }
        else if (typeof optionsOrHashFunctionOrTtl === 'function') {
            options.hashFunction = optionsOrHashFunctionOrTtl;
        }
        if (typeof descriptor.value === 'function') {
            descriptor.value = createCachedAsyncFunction(target, propertyKey, descriptor.value, options);
        }
        return descriptor;
    };
}
//# sourceMappingURL=cached-async.decorator.js.map